/******************************************************************************
*
* AntiMafia
* Copyright (C) 2002 by [epidemiC] AntiMafia
* Newtella
* Copyright (C) 2000 by Newtella Inc.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
******************************************************************************/


#include "..\stdafx.h"
#include <stdio.h>
#include <stdlib.h>
#include "common.h"
#include "Object.h"
#include "String.h"
#include "HTTPHeader.h"

HTTPHeader::HTTPHeader()
{
	isResponse = FALSE;
	code = 0;
	rangeStart = 0;
	rangeEnd = 0;
	contentLength = 0;
	contentRangeStart = 0;
	contentRangeEnd = 0;
	contentRangeTotal = 0;
}

HTTPHeader::~HTTPHeader()
{
	int fgf=345;
}

BOOL HTTPHeader::Parse()
{
	String line, word;

	if (isResponse)
	{
		if (!header.GetLine(0, line))
			return(FALSE);
		String scode;
		if (!line.GetWord(1, scode))
			return(FALSE);
		code = atoi(scode);
		line.GetWord(2, reason); // FIXME another stupid gnutella compatibility stuff
		for (int i=1; header.GetLine(i, line); i++)
		{
			if (!line.GetWord(0, word))
				continue;
			if (word == String("Server:"))
				line.GetWord(1, server);
			if (word == String("Content-Type:"))
				line.GetWord(1, contentType);
			if (word == String("Content-Length:"))
			{
				String slength;
				if (line.GetWord(1, slength))
					contentLength = atoi(slength);
			}
			if (word == String("Content-Range:"))
			{
				String slength;
				if (line.GetWord(1, slength))
				{
					String scl = slength.RightOfLast('/');
					contentLength = atoi(scl);
				}
			}
			// Here comes the compatibility stuff for the braindead original Gnutella client
			if (word == String("Content-range:"))
			{
				String slength;
				if (line.GetWord(1, slength))
				{
					String scl = slength.RightOfLast('/');
					contentLength = atoi(scl);
				}
			}
			if (!contentType.GetLength() && word.Left(13) == String("Content-type:"))
				contentType = word.Right(word.GetLength()-13);
			if (!contentLength && word.Left(15) == String("Content-length:"))
			{
				String slength = word.Right(word.GetLength()-15);
				contentLength = atoi(slength);
			}
		}
	}
	else
	{
		if (!header.GetLine(0, line))
			return(FALSE);
		if (line.Count(' ') < 2)
			return(FALSE);
		if (!line.GetWord(0, type))
			return(FALSE);
		versionString = line.RightOfLast(' ');
		if (!versionString.GetLength())
			return(FALSE);
		fileName = line.RightOf(' ');
		fileName = fileName.LeftOfLast(' ');
		if (!fileName.GetLength())
			return(FALSE);
		for (int i=1; header.GetLine(i, line); i++)
		{
			if (!line.GetWord(0, word))
				continue;
			if (word == String("User-Agent:"))
				line.GetWord(1, userAgent);
			if (word == String("Range:"))
			{
				String srange;
				if (line.GetWord(1, srange))
				{
					srange = srange.RightOf('=');
					String sstart = srange.LeftOf('-');
					String send = srange.RightOf('-');
					rangeStart = atoi(sstart);
					rangeEnd = atoi(send);
				}
			}
		}
	}

	return(TRUE);
}

BOOL HTTPHeader::Build()
{
	if (isResponse)
	{
		header = "HTTP ";
		char codestr[MAX_TEMPBUF_LEN];
		sprintf(codestr, "%d", code);
		header += codestr;
		header += " ";
		header += reason;
		header += "\r\n";
		if (server.GetLength())
		{
			header += "Server: ";
			header += server;
			header += "\r\n";
		}
		if (contentType.GetLength())
		{
			header += "Content-Type: ";
			header += contentType;
			header += "\r\n";
		}
		if (contentLength)
		{
			header += "Content-Length: ";
			char lengthstr[MAX_TEMPBUF_LEN];
			sprintf(lengthstr, "%d", contentLength);
			header += lengthstr;
			header += "\r\n";
		}
		if (contentRangeStart || contentRangeEnd)
		{
			header += "Content-Range: ";
			char rangestr[MAX_TEMPBUF_LEN];
			sprintf(rangestr, "bytes=%d-%d/%d", contentRangeStart, contentRangeEnd, contentRangeTotal);
			header += rangestr;
			header += "\r\n";
		}
		header += "\r\n";
	}
	else
	{
		header = type;
		header += " ";
		header += fileName;
		header += " ";
		header += versionString;
		header += "\r\n";
		if (userAgent.GetLength())
		{
			header += "User-Agent: ";
			header += userAgent;
			header += "\r\n";
		}
		if (range.GetLength())
		{
			header += "Range: ";
			header += range;
			header += "\r\n";
		}
		header += "\r\n";
	}
	return(TRUE);
}

