/******************************************************************************
*
* AntiMafia
* Copyright (C) 2002 by [epidemiC] AntiMafia
* Newtella
* Copyright (C) 2000 by Newtella Inc.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
******************************************************************************/


#include "..\stdafx.h"
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "common.h"
#include "Buffer.h"
#include "Object.h"
#include "String.h"
#include "List.h"
#include "FileWorker.h"
#include "Connection.h"
#include "GnutellaWorker.h"
#include "Base.h"

List::List()
{
	first = NULL; last = NULL; current = NULL;
	count = 0; maxNodes = 0;
}

List::~List()
{
	int i ;
	if (i = CheckMemErrors())
	{
		int i = 0 ;
	}
	Reset_Private();
}

BOOL List::Reset()
{
	return(Reset_Private());
}

BOOL List::Reset_Private()
{

	while (first) Remove(first->data) ;

	return TRUE ;

}

BOOL List::Add(int desc, Object *data)
{
	int rval;

	Node *node = new Node;
	rval = node->Set(desc, data);
	if (!rval)
	{
		delete data ;
		delete node ;
		return(FALSE);
	}

	if (first)
	{
		node->previous = last;
		node->next = NULL;
		last->next = node;
		last = node;
	}
	else
	{
		// List is empty
		first = node; last = node;
		node->next = NULL; node->previous = NULL;
	}
	count++;

	if (maxNodes && count > maxNodes) Remove(first->data);

	return(TRUE);
}

BOOL List::Add(Object *data)
{
	return(Add(0, data));
}

BOOL List::Remove(Object *ob)
{

	if (!first || !ob) return(FALSE);

	DWORD *dw, d ;

	Unlink(ob) ;

	dw = (DWORD*)ob ;
	d = *dw ;
	if (d > 0x400 && d < 0x40000000)
	{
		if (ob) delete ob ;
	}
	else
	{
		d = 0 ;
	}

	return(TRUE);

}


int List::CheckMemErrors()
{

	Node *working, *next ;
	DWORD *dw, d, i = 0 ;

	if (!first) return 0 ;

	working = first ;

	do
	{
		next = working->next ;
		dw = (DWORD*)working->data ;
		d = *dw ;
		if (d < 0x4000 || d > 0x40000000)
		{
			i++ ;
		}
		working = next ;
	} while (working) ;

	return i ;

}





BOOL List::SetMaxNodes(unsigned int max)
{
	maxNodes = max;
	while (count > maxNodes)
		Remove(first->data);
	return(TRUE);
}

BOOL List::Unlink(Object *ob)
{

	if (!first || !ob) return(FALSE);

	Node *working = first, *next;
	do
	{
		next = working->next;
		if (working->data == ob)
		{
			if (working->previous)
				working->previous->next = working->next;
			if (working->next)
				working->next->previous = working->previous;
			if (working == first)
				first = working->next;
			if (working == last)
				last = working->previous;
			if (working == current)
				current = working->next;
			delete working ;
			count-- ;
		}
		working = next ;
	} while (working) ;
	return(TRUE);

}

Object *List::GetData(int desc)
{
	if (!first)
		return(NULL);

	Node *working = first;
	do
	{
		if (working->descriptor == desc)
			return(working->data);
	} while (working = working->next);
	return(NULL);
}

int List::GetPreviousDescriptor()
{
	if (!first) return -1 ;

	if (current && current->previous)
	{
		return current->previous->descriptor ;
	}

	return last->descriptor ;
}

Object *List::operator[](int i)
{
	if (!first)
		return(NULL);

	Node *working = first;
	do
	{
		if (working->descriptor == i)
			return(working->data);
	} while (working = working->next);
	return(NULL);
}

BOOL List::SetCursor(int i)
{
	if (!first)
		return(NULL);

	Node *working = first;
	while (working)
	{
		if (i == 0)
		{
			current = working;
			return(TRUE);
		}
		i--;
		working = working->next;
	}
	return(FALSE);
}

Object *List::GetFirst(BOOL spool)
{
	if (!first)
		return(NULL);

	current = first;
	Object *r = current->data;
	if (spool)
		current = current->next;
	return(r);
}

Object *List::GetNext(BOOL spool)
{
	if (!current)
		return(NULL);

	Object *r = current->data;
	if (spool)
	{
		current = current->next;
	}
	return(r);
}

Object *List::GetLast(BOOL spool)
{
	if (!first)
		return(NULL);

	current = last;
	Object *r = current->data;
	if (spool)
		current = current->previous;
	return(r);
}

Object *List::GetPrevious(BOOL spool)
{
	if (!current)
		return(NULL);

	Object *r = current->data;
	if (spool)
	{
		current = current->previous;
	}
	return(r);
}

Object *List::GetNextByDescriptor(int desc, BOOL spool)
{
	if (!current)
		return(NULL);

	Node *working = current;
	do
	{
		if (working->descriptor == desc)
		{
			if (spool)
				current = working->next;
			return(working->data);
		}
	} while (working = working->next);
	return(NULL);
}

Object *List::GetRandom()
{
	if (!first)
		return(NULL);

	int n = rand() % count;
	Node *r = first;
	for (int i=n; i>0; i--)
		r = r->next;

	return(r->data);
}

BOOL List::Dump(FILE *file, int n)
{
	if (n)
	{
		Object *object;
		for (int i=0; i<n; i++)
		{
			object = GetRandom();
			if (!object)
				return(TRUE);
			object->Dump(file);
		}
	}
	else
	{
		Node *working = first;
		while (working)
		{
			if (working->data)
				working->data->Dump(file);
			working = working->next;
		}
	}

	return(TRUE);
}

BOOL List::Serialize(FILE *file, BOOL store)
{
	const int serializeVersion=LIST_SERIALIZEVERSION;
	int fileVersion;

	HostInfo *hi=NULL;
	FileInfo *fi=NULL;
	Connection *con=NULL;
	if (store)
	{
		STORE_VAR(serializeVersion);

		STORE_VAR(count);
		Node *working = first;
		while (working)
		{
			STORE_VAR(working->descriptor);
			STORE_SUBCLASS((*working->data));
			working = working->next;
		}
		return(TRUE);
	}
	else
	{
		int newcount, i, desc;
		CHECK_VERSION();

		LOAD_VAR(newcount);
		if (dt == DATATYPE_HOSTINFO)
		{
			for (i=0; i<newcount; i++)
			{
				hi = new HostInfo;
				LOAD_VAR(desc);
				LOAD_SUBCLASS((*hi));
				Add(desc, hi);
			}
		}
		else if (dt == DATATYPE_FILEINFO)
		{
			for (i=0; i<newcount; i++)
			{
				fi = new FileInfo;
				LOAD_VAR(desc);
				LOAD_SUBCLASS((*fi));
				Add(desc, fi);
			}
		}
		else if (dt == DATATYPE_CONNECTION)
		{
			for (i=0; i<newcount; i++)
			{
				con = new Connection;
				LOAD_VAR(desc);
				LOAD_SUBCLASS((*con));
				Add(desc, con);
			}
		}
	}
	return(TRUE);
serializeFailed:
	if (!store)
	{
		if (dt == DATATYPE_HOSTINFO)
		{
			if (hi)
				delete hi;
		}
		else if (dt == DATATYPE_FILEINFO)
		{
			if (fi)
				delete fi;
		}
		Reset();
	}
	return(FALSE);
}

BOOL List::SetDataType(DataType d)
{
	dt = d;
	return(TRUE);
}





















Node::Node()
{
	data = NULL;
	descriptor = NODE_UNUSED;
	next = NULL;
	previous = NULL;
}

Node::~Node()
{
	descriptor = NODE_UNUSED;
	next = NULL;
	previous = NULL;
}

Node::Node(int desc, Object *data)
{
	data = NULL;
	descriptor = NODE_UNUSED;
	next = NULL;
	previous = NULL;
	Set_Private(desc, data);
}

BOOL Node::Set(int desc, Object *data)
{
	return(Set_Private(desc, data));
}

BOOL Node::Set_Private(int desc, Object *newdata)
{
	if (data) delete data;

	data = NULL;
	descriptor = NODE_UNUSED;
	next = NULL;
	previous = NULL;

	if (!newdata) return(FALSE);

	data = newdata;
	descriptor = desc;

	return(TRUE);
}

